-- LOCALISATION

local AddonName, AddonTable = ...
local L = AddonTable.Localize
local AddonTitle = select(2, GetAddOnInfo(AddonName))
local PlainAddonTitle = AddonTitle:gsub("|c........", ""):gsub("|r", "")

-- BACKPACK TEXT

local FBSBackpackLabel = MainMenuBarBackpackButton:CreateFontString( "BagFreeSlotLabel", "OVERLAY", "GameTooltipText")
FBSBackpackLabel:SetFont("Fonts\\ARIALN.TTF", 14, "THINOUTLINE")
FBSBackpackLabel:SetPoint("BOTTOM", 0, 3)
--FBSBackpackLabel:SetTextColor(1,1,1,1)

-- MOVEABLE FRAME

local freeslotsframe = CreateFrame("Frame", "FreeBagSpacesFrame", UIParent)
freeslotsframe:SetSize(16, 16) -- WxH
freeslotsframe:SetPoint("CENTER")
freeslotsframe:SetClampedToScreen(true)
--freeslotsframe:EnableMouse(true)
freeslotsframe:SetMovable(true)
freeslotsframe:RegisterForDrag("LeftButton")
freeslotsframe:SetScript("OnDragStart", function(self) self:StartMoving() end)
freeslotsframe:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
freeslotsframe:SetUserPlaced(true)
local bg = freeslotsframe:CreateTexture()
bg:SetAllPoints(freeslotsframe)
--bg:SetColorTexture(0, 0, 0, 0.5)

local BagIcon = freeslotsframe:CreateTexture(nil, "ARTWORK")
BagIcon:SetPoint("TOPLEFT", 16, -16)
BagIcon:SetSize(20,20)
BagIcon:SetTexture("Interface\\AddOns\\FreeBagSpaces\\Textures\\BagIcon.tga")

local freeslots = freeslotsframe:CreateFontString(nil, nil, "GameFontHighlight")
freeslots:SetPoint("LEFT", BagIcon, "RIGHT", 8, 0)
freeslots:SetFont("Fonts\\FRIZQT__.ttf", 12)

local TotalFreeSlotsFrame = CreateFrame("Frame")
TotalFreeSlotsFrame:RegisterEvent("BAG_UPDATE")
TotalFreeSlotsFrame:RegisterEvent("BAG_UPDATE_COOLDOWN")
TotalFreeSlotsFrame:RegisterEvent("BAG_UPDATE_DELAYED")
TotalFreeSlotsFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
TotalFreeSlotsFrame:RegisterEvent("ADDON_LOADED")
TotalFreeSlotsFrame:RegisterEvent("UNIT_INVENTORY_CHANGED")

-- FRAME EVENT SCRIPT

local function eventHandler(self, event, arg1)

if event == "ADDON_LOADED" and arg1 == "FreeBagSpaces" then

SlashCmdList["FBSCONFIG"] = fbsconfig;
	SLASH_FBSCONFIG1 = "/fbs"

if FBSConfig == "" or FBSConfig == nil then
	FBSConfig = {
		["ShowFrame"] = "YES",
		["FrameLock"] = "NO",
		["ShowTotal"] = "YES",
		["Backpack"] = "YES",
		["AmmoCount"] = "YES",
		["UseComma"] = "NO",
		["FontSize"] = "12"
	}
end

if FBSConfig.FrameLock == nil then FBSConfig.FrameLock = 'NO' end
if FBSConfig.ShowTotal == nil then FBSConfig.ShowTotal = 'YES' end
if FBSConfig.Backpack == nil then FBSConfig.Backpack = 'YES' end
if FBSConfig.AmmoCount == nil then FBSConfig.AmmoCount = 'YES' end
if FBSConfig.UseComma == nil then FBSConfig.UseComma = 'NO' end
if FBSConfig.FontSize == nil then FBSConfig.FontSize = '12' end

if FBSConfig.ShowFrame == "YES" then freeslotsframe:Show() elseif FBSConfig.ShowFrame == "NO" then freeslotsframe:Hide() end
if FBSConfig.Backpack == "NO" then FBSBackpackLabel:SetTextColor(1,1,1,0) elseif FBSConfig.Backpack == "YES" then FBSBackpackLabel:SetTextColor(1,1,1,1) end

if FBSConfig.FrameLock == "NO" then
	bg:SetColorTexture(0, 0, 0, 0.5)
	freeslotsframe:EnableMouse(true)
elseif FBSConfig.FrameLock == "YES" then
	bg:SetColorTexture(0, 0, 0, 0)
	freeslotsframe:EnableMouse(false)
end

DEFAULT_CHAT_FRAME:AddMessage("|cFF00AA00[" .. L["Free Bag Spaces"] .. "]|r v" .. GetAddOnMetadata("FreeBagSpaces", "Version") .. " loaded.")

UpdateFonts()

elseif event == "BAG_UPDATE" or event == "BAG_UPDATE_COOLDOWN" or event == "BAG_UPDATE_DELAYED" or event == "PLAYER_ENTERING_WORLD" or event == "UNIT_INVENTORY_CHANGED" then

UpdateBags()
GetAmmo()

end

end

-- EVENT HANDLER

TotalFreeSlotsFrame:SetScript("OnEvent", eventHandler)

-- START OF OPTIONS WINDOW

local FBSIOFrame = CreateFrame("Frame")
FBSIOFrame.name = L["Free Bag Spaces"]

local lblTitle = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
lblTitle:SetFont("Fonts\\FRIZQT__.TTF", 12)
lblTitle:SetPoint("TOPLEFT", FBSIOFrame, "TOPLEFT", 12, -12)
lblTitle:SetText(L["Free Bag Spaces"] .. " v" .. GetAddOnMetadata("FreeBagSpaces","Version"))

-- Show Frame

local chkShowFrame = CreateFrame("CheckButton", nil, FBSIOFrame, "OptionsBaseCheckButtonTemplate")
chkShowFrame:SetPoint("TOPLEFT", lblTitle, "BOTTOMLEFT", 0, -16)

chkShowFrame:SetScript("OnUpdate", function(frame)
	if FBSConfig.ShowFrame == "YES" then
		chkShowFrame:SetChecked(true)
	elseif FBSConfig.ShowFrame == "NO" then
		chkShowFrame:SetChecked(false)
	end
end)

chkShowFrame:SetScript("OnClick", function(frame)
local tick = frame:GetChecked()

	if tick == false then
		FBSConfig.ShowFrame = 'NO'
		freeslotsframe:Hide()
	elseif tick == true then
		FBSConfig.ShowFrame = 'YES'
		freeslotsframe:Show()
	end
end)

local chkShowFrameText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
chkShowFrameText:SetPoint("LEFT", chkShowFrame, "RIGHT", 0, 1)
chkShowFrameText:SetText(L["Show Frame"])

-- Lock Frame

local chkLockFrame = CreateFrame("CheckButton", nil, FBSIOFrame, "OptionsBaseCheckButtonTemplate")
chkLockFrame:SetPoint("TOPLEFT", chkShowFrame, "BOTTOMLEFT", 32, -16)

chkLockFrame:SetScript("OnUpdate", function(frame)
	if FBSConfig.FrameLock == "YES" then
		chkLockFrame:SetChecked(true)
	elseif FBSConfig.FrameLock == "NO" then
		chkLockFrame:SetChecked(false)
	end
end)

chkLockFrame:SetScript("OnClick", function(frame)
local tick = frame:GetChecked()

	if tick == false then
		FBSConfig.FrameLock = 'NO'
		bg:SetColorTexture(0, 0, 0, 0.5)
		freeslotsframe:EnableMouse(true)
	elseif tick == true then
		FBSConfig.FrameLock = 'YES'
		bg:SetColorTexture(0, 0, 0, 0)
		freeslotsframe:EnableMouse(false)
	end
end)

local chkLockFrameText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
chkLockFrameText:SetPoint("LEFT", chkLockFrame, "RIGHT", 0, 1)
chkLockFrameText:SetText(L["Lock Frame"])

-- Show Total Slots

local chkShowTotalSlots = CreateFrame("CheckButton", nil, FBSIOFrame, "OptionsBaseCheckButtonTemplate")
chkShowTotalSlots:SetPoint("TOPLEFT", chkLockFrame, "BOTTOMLEFT", -32, -16)

chkShowTotalSlots:SetScript("OnUpdate", function(frame)
	if FBSConfig.ShowTotal == "YES" then
		chkShowTotalSlots:SetChecked(true)
	elseif FBSConfig.ShowTotal == "NO" then
		chkShowTotalSlots:SetChecked(false)
	end
end)

chkShowTotalSlots:SetScript("OnClick", function(frame)
local tick = frame:GetChecked()

	if tick == false then
		FBSConfig.ShowTotal = 'NO'
	elseif tick == true then
		FBSConfig.ShowTotal = 'YES'
	end
	
	UpdateBags()
end)

local chkShowTotalSlotsText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
chkShowTotalSlotsText:SetPoint("LEFT", chkShowTotalSlots, "RIGHT", 0, 1)
chkShowTotalSlotsText:SetText(L["Show Total Number Of Slots"])

-- Backpack Text

local chkShowBackpack = CreateFrame("CheckButton", nil, FBSIOFrame, "OptionsBaseCheckButtonTemplate")
chkShowBackpack:SetPoint("TOPLEFT", chkShowTotalSlots, "BOTTOMLEFT", 0, -16)

chkShowBackpack:SetScript("OnUpdate", function(frame)
	if FBSConfig.Backpack == "YES" then
		chkShowBackpack:SetChecked(true)
	elseif FBSConfig.Backpack == "NO" then
		chkShowBackpack:SetChecked(false)
	end
end)

chkShowBackpack:SetScript("OnClick", function(frame)
local tick = frame:GetChecked()

	if tick == false then
		FBSConfig.Backpack = 'NO'
		FBSBackpackLabel:SetTextColor(1,1,1,0)
	elseif tick == true then
		FBSConfig.Backpack = 'YES'
		FBSBackpackLabel:SetTextColor(1,1,1,1)
	end
end)

local chkShowBackpackText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
chkShowBackpackText:SetPoint("LEFT", chkShowBackpack, "RIGHT", 0, 1)
chkShowBackpackText:SetText(L["Show Text On Backpack"])

-- Ammo Count

local chkShowAmmoCount = CreateFrame("CheckButton", nil, FBSIOFrame, "OptionsBaseCheckButtonTemplate")
chkShowAmmoCount:SetPoint("TOPLEFT", chkShowBackpack, "BOTTOMLEFT", 0, -16)

chkShowAmmoCount:SetScript("OnUpdate", function(frame)
	if FBSConfig.AmmoCount == "YES" then
		chkShowAmmoCount:SetChecked(true)
	elseif FBSConfig.AmmoCount == "NO" then
		chkShowAmmoCount:SetChecked(false)
	end
end)

chkShowAmmoCount:SetScript("OnClick", function(frame)
local tick = frame:GetChecked()

	if tick == false then
		FBSConfig.AmmoCount = 'NO'
	elseif tick == true then
		FBSConfig.AmmoCount = 'YES'
	end
	
	UpdateBags()
end)

local chkShowAmmoCountText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
chkShowAmmoCountText:SetPoint("LEFT", chkShowAmmoCount, "RIGHT", 0, 1)
chkShowAmmoCountText:SetText(L["Show remaining ammo (Hunters only)"])

-- Comma Separation

local chkCommaSeparation = CreateFrame("CheckButton", nil, FBSIOFrame, "OptionsBaseCheckButtonTemplate")
chkCommaSeparation:SetPoint("TOPLEFT", chkShowAmmoCount, "BOTTOMLEFT", 0, -16)

chkCommaSeparation:SetScript("OnUpdate", function(frame)
	if FBSConfig.UseComma == "YES" then
		chkCommaSeparation:SetChecked(true)
	elseif FBSConfig.UseComma == "NO" then
		chkCommaSeparation:SetChecked(false)
	end
end)

chkCommaSeparation:SetScript("OnClick", function(frame)
local tick = frame:GetChecked()

	if tick == false then
		FBSConfig.UseComma = 'NO'
	elseif tick == true then
		FBSConfig.UseComma = 'YES'
	end
	
	UpdateBags()
end)

local chkCommaSeparationText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
chkCommaSeparationText:SetPoint("LEFT", chkCommaSeparation, "RIGHT", 0, 1)
chkCommaSeparationText:SetText(L["Number Formatting"])

-- Font Size

local lblIOFontSizeText = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
lblIOFontSizeText:SetPoint("TOPLEFT", chkCommaSeparation, "BOTTOMLEFT", 0, -16)
lblIOFontSizeText:SetText(L["Font size:"])

--local ddFontSize = CreateFrame("Frame", "GTFontSize", FBSIOFrame, "UIDropDownMenuTemplate")

-- Create the dropdown, and configure its appearance
local ddFBSFontSize = CreateFrame("FRAME", "FBSFontSize", FBSIOFrame, "UIDropDownMenuTemplate")
ddFBSFontSize:SetPoint("LEFT", lblIOFontSizeText, "RIGHT", 0, 1)
if GetLocale() == "frFR" then
	UIDropDownMenu_SetWidth(ddFBSFontSize, 128)
else
	UIDropDownMenu_SetWidth(ddFBSFontSize, 96)
end
UIDropDownMenu_SetText(ddFBSFontSize, L["Select One"])

-- Create and bind the initialization function to the dropdown menu
UIDropDownMenu_Initialize(ddFBSFontSize, function(self, level, menuList)
local info = UIDropDownMenu_CreateInfo()
	info.func = self.SetValue
	info.text, info.arg1 = "12", 12
	UIDropDownMenu_AddButton(info)
	info.text, info.arg1 = "14", 14
	UIDropDownMenu_AddButton(info)
	info.text, info.arg1 = "16", 16
	UIDropDownMenu_AddButton(info)
	info.text, info.arg1 = "18", 18
	UIDropDownMenu_AddButton(info)
	info.text, info.arg1 = "20", 20
	UIDropDownMenu_AddButton(info)
end)

-- Implement the function to change the font size
function ddFBSFontSize:SetValue(newValue)
	FBSConfig.FontSize = newValue
	DEFAULT_CHAT_FRAME:AddMessage("|cFF00AA00[" .. L["Free Bag Spaces"] .. "]|r " .. L["Font size changed to"] .. " " .. newValue .. ".")
	UpdateBags()
	GetAmmo()
	UpdateFonts()
	-- Update the text; if we merely wanted it to display newValue, we would not need to do this
	UIDropDownMenu_SetText(ddFBSFontSize, FBSConfig.FontSize)
	-- Because this is called from a sub-menu, only that menu level is closed by default.
	-- Close the entire menu with this next call
	CloseDropDownMenus()
end

-- Discord Info

local DiscordLogo = FBSIOFrame:CreateTexture(nil, "ARTWORK")
DiscordLogo:SetPoint("TOPLEFT", chkCommaSeparation, "BOTTOMLEFT", 0, -52)
DiscordLogo:SetSize(16, 16)
DiscordLogo:SetTexture("Interface\\AddOns\\FreeBagSpaces\\Textures\\DiscordLogo.tga")

local lblDiscord = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
lblDiscord:SetPoint("LEFT", DiscordLogo, "RIGHT", 4, 0)
lblDiscord:SetText("|cFFEFC502https://discord.gg/BGKnpza|r")

-- Twitter Info

local TwitterLogo = FBSIOFrame:CreateTexture(nil, "ARTWORK")
TwitterLogo:SetPoint("TOPLEFT", DiscordLogo, "BOTTOMLEFT", 0, -8)
TwitterLogo:SetSize(16, 16)
TwitterLogo:SetTexture("Interface\\AddOns\\FreeBagSpaces\\Textures\\TwitterLogo.tga")

local lblTwitter = FBSIOFrame:CreateFontString(nil, nil, "GameFontHighlight")
lblTwitter:SetPoint("LEFT", TwitterLogo, "RIGHT", 4, 0)
lblTwitter:SetText("|cFFEFC502@GeodesicDragon|r")

-- END OF CONFIG WINDOW

InterfaceOptions_AddCategory(FBSIOFrame)

function UpdateBags()

localizedClass, englishClass, classIndex = UnitClass("player")

numberOfFreeSlotsBagZero, BagTypeZero = GetContainerNumFreeSlots(0)
numberOfFreeSlotsBagOne, BagTypeOne = GetContainerNumFreeSlots(1)
numberOfFreeSlotsBagTwo, BagTypeTwo = GetContainerNumFreeSlots(2)
numberOfFreeSlotsBagThree, BagTypeThree = GetContainerNumFreeSlots(3)
numberOfFreeSlotsBagFour, BagTypeFour = GetContainerNumFreeSlots(4)

local totalFreeSlots = numberOfFreeSlotsBagZero + numberOfFreeSlotsBagOne + numberOfFreeSlotsBagTwo + numberOfFreeSlotsBagThree + numberOfFreeSlotsBagFour

numberOfTotalSlotsBagZero = GetContainerNumSlots(0);
numberOfTotalSlotsBagOne = GetContainerNumSlots(1);
numberOfTotalSlotsBagTwo = GetContainerNumSlots(2);
numberOfTotalSlotsBagThree = GetContainerNumSlots(3);
numberOfTotalSlotsBagFour = GetContainerNumSlots(4);

local CurrentAmmo = GetAmmo()

local totalAvailableSlots = numberOfTotalSlotsBagZero + numberOfTotalSlotsBagOne + numberOfTotalSlotsBagTwo + numberOfTotalSlotsBagThree + numberOfTotalSlotsBagFour

if BagTypeZero ~= 0 then
	totalFreeSlots = (totalFreeSlots - numberOfFreeSlotsBagZero)
	totalAvailableSlots = (totalAvailableSlots - numberOfTotalSlotsBagZero)
end

if BagTypeOne ~= 0 then
	totalFreeSlots = (totalFreeSlots - numberOfFreeSlotsBagOne)
	totalAvailableSlots = (totalAvailableSlots - numberOfTotalSlotsBagOne)
end

if BagTypeTwo ~= 0 then
	totalFreeSlots = (totalFreeSlots - numberOfFreeSlotsBagTwo)
	totalAvailableSlots = (totalAvailableSlots - numberOfTotalSlotsBagTwo)
end

if BagTypeThree ~= 0 then
	totalFreeSlots = (totalFreeSlots - numberOfFreeSlotsBagThree)
	totalAvailableSlots = (totalAvailableSlots - numberOfTotalSlotsBagThree)
end

if BagTypeFour ~= 0 then
	totalFreeSlots = (totalFreeSlots - numberOfFreeSlotsBagFour)
	totalAvailableSlots = (totalAvailableSlots - numberOfTotalSlotsBagFour)
end

--freeslots:SetText(BAGSLOTTEXT .. ": " .. tonumber(totalFreeSlots) .. "/" .. tonumber(totalAvailableSlots))

if FBSConfig.ShowTotal == "YES" then
	if classIndex == 3 then
		if FBSConfig.AmmoCount == "YES" then
			freeslots:SetText(tonumber(totalFreeSlots) .. "/" .. tonumber(totalAvailableSlots) .. " | " .. INVTYPE_AMMO .. ": " .. CurrentAmmo)
		elseif FBSConfig.AmmoCount == "NO" then
			freeslots:SetText(tonumber(totalFreeSlots) .. "/" .. tonumber(totalAvailableSlots))
		end
	elseif classIndex ~= 3 then
		freeslots:SetText(tonumber(totalFreeSlots) .. "/" .. tonumber(totalAvailableSlots))
	end
elseif FBSConfig.ShowTotal == "NO" then
	if classIndex == 3 then
		if FBSConfig.AmmoCount == "YES" then
			freeslots:SetText(tonumber(totalFreeSlots) .. " | " .. INVTYPE_AMMO .. ": " .. CurrentAmmo)
		elseif FBSConfig.AmmoCount == "NO" then
			freeslots:SetText(tonumber(totalFreeSlots))
		end
	elseif classIndex ~= 3 then
		freeslots:SetText(tonumber(totalFreeSlots))
	end
end

FBSBackpackLabel:SetFormattedText("(%d)", totalFreeSlots)

end

function GetAmmo()

localizedClass, englishClass, classIndex = UnitClass("player")

if classIndex ~= 3 then
	return
elseif classIndex == 3 then
	local ammoSlot = GetInventorySlotInfo("AmmoSlot")
	local ammoCount = GetInventoryItemCount("player", ammoSlot)

	if ((ammoCount == 1) and (not GetInventoryItemTexture("player", ammoSlot))) then
		ammoCount = 0;
	end

	if ammoCount >= 200 then
		if FBSConfig.UseComma == "YES" then
			return(comma_val(ammoCount))
		elseif FBSConfig.UseComma == "NO" then
			return(ammoCount)
		end
	elseif ammoCount >= 1 and ammoCount <= 199 then
		return(tostring("|cFFFFFF00" .. ammoCount .. "|r"))
	elseif ammoCount == 0 then
		return(tostring("|cFFFF0000" .. ammoCount .. "|r"))
	end
end

end

function comma_val(n)
	-- Found on http://lua-users.org/wiki/FormattingNumbers
	-- All credit goes to Richard Warburton (http://richard.warburton.it)
	
	local left,num,right = string.match(n,'^([^%d]*%d)(%d*)(.-)$')
	return left..(num:reverse():gsub('(%d%d%d)','%1,'):reverse())..right
end

function fbsconfig()

InterfaceOptionsFrame_OpenToCategory(L["Free Bag Spaces"])
InterfaceOptionsFrame_OpenToCategory(L["Free Bag Spaces"])

end

function UpdateFonts()
	BagIcon:SetSize(tonumber(FBSConfig.FontSize), tonumber(FBSConfig.FontSize))
	freeslots:SetFont("Fonts\\FRIZQT__.ttf", tonumber(FBSConfig.FontSize))
end